#
# This file is part of App-Cme
#
# This software is Copyright (c) 2014-2022 by Dominique Dumont <ddumont@cpan.org>.
#
# This is free software, licensed under:
#
#   The GNU Lesser General Public License, Version 2.1, February 1999
#

# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.037
use strict;
use warnings;

use Module::Build 0.34;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.34"
  },
  "configure_requires" => {
    "Module::Build" => "0.34"
  },
  "dist_abstract" => "Configuration data checker or editor based on Config::Model",
  "dist_author" => [
    "Dominique Dumont"
  ],
  "dist_name" => "App-Cme",
  "dist_version" => "1.049",
  "license" => "lgpl",
  "module_name" => "App::Cme",
  "recommends" => {
    "Config::Model::TkUI" => "1.370"
  },
  "recursive_test_files" => 1,
  "requires" => {
    "App::Cmd::Setup" => 0,
    "Config::Model" => "2.156",
    "Config::Model::FuseUI" => 0,
    "Config::Model::Lister" => 0,
    "Config::Model::ObjTreeScanner" => 0,
    "Config::Model::SimpleUI" => 0,
    "Config::Model::TermUI" => 0,
    "Config::Model::Utils::GenClassPod" => 0,
    "Data::Dumper" => 0,
    "Encode" => 0,
    "File::HomeDir" => 0,
    "JSON" => 0,
    "Log::Log4perl" => 0,
    "Path::Tiny" => "0.125",
    "Pod::POM" => 0,
    "Pod::POM::View::Text" => 0,
    "Scalar::Util" => 0,
    "Tie::Hash" => 0,
    "YAML::PP" => 0,
    "perl" => "5.020"
  },
  "script_files" => [
    "bin/cme"
  ],
  "test_requires" => {
    "App::Cmd::Tester" => 0,
    "Config::Model::Backend::Yaml" => "2.133",
    "Config::Model::Tester::Setup" => 0,
    "Config::Model::TkUI" => "1.370",
    "English" => 0,
    "File::Spec" => 0,
    "Path::Tiny" => "0.125",
    "Term::ANSIColor" => "2.01",
    "Test::Differences" => 0,
    "Test::File::Contents" => 0,
    "Test::More" => 0,
    "Test::Perl::Critic" => 0,
    "Test::Pod" => "1.00"
  }
);


my %fallback_build_requires = (
  "App::Cmd::Tester" => 0,
  "Config::Model::Backend::Yaml" => "2.133",
  "Config::Model::Tester::Setup" => 0,
  "Config::Model::TkUI" => "1.370",
  "English" => 0,
  "File::Spec" => 0,
  "Module::Build" => "0.34",
  "Path::Tiny" => "0.125",
  "Term::ANSIColor" => "2.01",
  "Test::Differences" => 0,
  "Test::File::Contents" => 0,
  "Test::More" => 0,
  "Test::Perl::Critic" => 0,
  "Test::Pod" => "1.00"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
